@@ -9,6 +9,9 @@ import pytest
 import sys
 import textwrap
 import unittest
+import random
+
+from importlib.metadata import version
 
 from contextlib import contextmanager
 
@@ -18,6 +21,7 @@ from IPython.core import completer
 from IPython.utils.tempdir import TemporaryDirectory, TemporaryWorkingDirectory
 from IPython.utils.generics import complete_object
 from IPython.testing import decorators as dec
+from IPython.core.latex_symbols import latex_symbols
 
 from IPython.core.completer import (
     Completion,
@@ -28,12 +32,29 @@ from IPython.core.completer import (
     completion_matcher,
     SimpleCompletion,
     CompletionContext,
+    _unicode_name_compute,
+    _UNICODE_RANGES,
 )
 
+from packaging.version import parse
+
+
+@contextmanager
+def jedi_status(status: bool):
+    completer = get_ipython().Completer
+    try:
+        old = completer.use_jedi
+        completer.use_jedi = status
+        yield
+    finally:
+        completer.use_jedi = old
+
+
 # -----------------------------------------------------------------------------
 # Test functions
 # -----------------------------------------------------------------------------
 
+
 def recompute_unicode_ranges():
     """
     utility to recompute the largest unicode range without any characters
@@ -42,8 +63,9 @@ def recompute_unicode_ranges():
     """
     import itertools
     import unicodedata
+
     valid = []
-    for c in range(0,0x10FFFF + 1):
+    for c in range(0, 0x10FFFF + 1):
         try:
             unicodedata.name(chr(c))
         except ValueError:
@@ -58,22 +80,27 @@ def recompute_unicode_ranges():
     rg = list(ranges(valid))
     lens = []
     gap_lens = []
-    pstart, pstop = 0,0
+    _pstart, pstop = 0, 0
     for start, stop in rg:
-        lens.append(stop-start)
-        gap_lens.append((start - pstop, hex(pstop), hex(start), f'{round((start - pstop)/0xe01f0*100)}%'))
-        pstart, pstop = start, stop
+        lens.append(stop - start)
+        gap_lens.append(
+            (
+                start - pstop,
+                hex(pstop + 1),
+                hex(start),
+                f"{round((start - pstop)/0xe01f0*100)}%",
+            )
+        )
+        _pstart, pstop = start, stop
 
     return sorted(gap_lens)[-1]
 
 
-
 def test_unicode_range():
     """
     Test that the ranges we test for unicode names give the same number of
     results than testing the full length.
     """
-    from IPython.core.completer import  _unicode_name_compute, _UNICODE_RANGES
 
     expected_list = _unicode_name_compute([(0, 0x110000)])
     test = _unicode_name_compute(_UNICODE_RANGES)
@@ -98,8 +125,8 @@ def test_unicode_range():
         """
     assert len_exp == len_test, message
 
-    # fail if new unicode symbols have been added. 
-    assert len_exp <= 143041, message
+    # fail if new unicode symbols have been added.
+    assert len_exp <= 143668, message
 
 
 @contextmanager
@@ -134,45 +161,45 @@ def custom_matchers(matchers):
         ip.Completer.custom_matchers.clear()
 
 
-def test_protect_filename():
-    if sys.platform == "win32":
-        pairs = [
-            ("abc", "abc"),
-            (" abc", '" abc"'),
-            ("a bc", '"a bc"'),
-            ("a  bc", '"a  bc"'),
-            ("  bc", '"  bc"'),
-        ]
-    else:
-        pairs = [
-            ("abc", "abc"),
-            (" abc", r"\ abc"),
-            ("a bc", r"a\ bc"),
-            ("a  bc", r"a\ \ bc"),
-            ("  bc", r"\ \ bc"),
-            # On posix, we also protect parens and other special characters.
-            ("a(bc", r"a\(bc"),
-            ("a)bc", r"a\)bc"),
-            ("a( )bc", r"a\(\ \)bc"),
-            ("a[1]bc", r"a\[1\]bc"),
-            ("a{1}bc", r"a\{1\}bc"),
-            ("a#bc", r"a\#bc"),
-            ("a?bc", r"a\?bc"),
-            ("a=bc", r"a\=bc"),
-            ("a\\bc", r"a\\bc"),
-            ("a|bc", r"a\|bc"),
-            ("a;bc", r"a\;bc"),
-            ("a:bc", r"a\:bc"),
-            ("a'bc", r"a\'bc"),
-            ("a*bc", r"a\*bc"),
-            ('a"bc', r"a\"bc"),
-            ("a^bc", r"a\^bc"),
-            ("a&bc", r"a\&bc"),
-        ]
-    # run the actual tests
-    for s1, s2 in pairs:
-        s1p = completer.protect_filename(s1)
-        assert s1p == s2
+if sys.platform == "win32":
+    pairs = [
+        ("abc", "abc"),
+        (" abc", '" abc"'),
+        ("a bc", '"a bc"'),
+        ("a  bc", '"a  bc"'),
+        ("  bc", '"  bc"'),
+    ]
+else:
+    pairs = [
+        ("abc", "abc"),
+        (" abc", r"\ abc"),
+        ("a bc", r"a\ bc"),
+        ("a  bc", r"a\ \ bc"),
+        ("  bc", r"\ \ bc"),
+        # On posix, we also protect parens and other special characters.
+        ("a(bc", r"a\(bc"),
+        ("a)bc", r"a\)bc"),
+        ("a( )bc", r"a\(\ \)bc"),
+        ("a[1]bc", r"a\[1\]bc"),
+        ("a{1}bc", r"a\{1\}bc"),
+        ("a#bc", r"a\#bc"),
+        ("a?bc", r"a\?bc"),
+        ("a=bc", r"a\=bc"),
+        ("a\\bc", r"a\\bc"),
+        ("a|bc", r"a\|bc"),
+        ("a;bc", r"a\;bc"),
+        ("a:bc", r"a\:bc"),
+        ("a'bc", r"a\'bc"),
+        ("a*bc", r"a\*bc"),
+        ('a"bc', r"a\"bc"),
+        ("a^bc", r"a\^bc"),
+        ("a&bc", r"a\&bc"),
+    ]
+
+
+@pytest.mark.parametrize("s1,expected", pairs)
+def test_protect_filename(s1, expected):
+    assert completer.protect_filename(s1) == expected
 
 
 def check_line_split(splitter, test_specs):
@@ -190,7 +217,7 @@ def test_line_split():
     # was at the end of part1.  So an empty part2 represents someone hitting
     # tab at the end of the line, the most common case.
     t = [
-        ("run some/scrip", "", "some/scrip"),
+        ("run some/script", "", "some/script"),
         ("run scripts/er", "ror.py foo", "scripts/er"),
         ("echo $HOM", "", "HOM"),
         ("print sys.pa", "", "sys.pa"),
@@ -283,8 +310,6 @@ class TestCompleter(unittest.TestCase):
             self.assertIsInstance(matches, list)
 
     def test_latex_completions(self):
-        from IPython.core.latex_symbols import latex_symbols
-        import random
 
         ip = get_ipython()
         # Test some random unicode symbols
@@ -428,6 +453,10 @@ class TestCompleter(unittest.TestCase):
             c = _("%ls foo")[0]
             self.assertEqual(c.text, escaped)
 
+    @pytest.mark.xfail(
+        sys.version_info.releaselevel in ("alpha",),
+        reason="Parso does not yet parse 3.13",
+    )
     def test_all_completions_dups(self):
         """
         Make sure the output of `IPCompleter.all_completions` does not have
@@ -444,8 +473,15 @@ class TestCompleter(unittest.TestCase):
                 matches = c.all_completions("TestClass.")
                 assert len(matches) > 2, (jedi_status, matches)
                 matches = c.all_completions("TestClass.a")
-                assert matches == ['TestClass.a', 'TestClass.a1'], jedi_status
+                if jedi_status:
+                    assert matches == ["TestClass.a", "TestClass.a1"], jedi_status
+                else:
+                    assert matches == [".a", ".a1"], jedi_status
 
+    @pytest.mark.xfail(
+        sys.version_info.releaselevel in ("alpha",),
+        reason="Parso does not yet parse 3.13",
+    )
     def test_jedi(self):
         """
         A couple of issue we had with Jedi
@@ -481,6 +517,10 @@ class TestCompleter(unittest.TestCase):
 
         _test_not_complete("does not mix types", 'a=(1,"foo");a[0].', "capitalize")
 
+    @pytest.mark.xfail(
+        sys.version_info.releaselevel in ("alpha",),
+        reason="Parso does not yet parse 3.13",
+    )
     def test_completion_have_signature(self):
         """
         Lets make sure jedi is capable of pulling out the signature of the function we are completing.
@@ -496,6 +536,10 @@ class TestCompleter(unittest.TestCase):
             "encoding" in c.signature
         ), "Signature of function was not found by completer"
 
+    @pytest.mark.xfail(
+        sys.version_info.releaselevel in ("alpha",),
+        reason="Parso does not yet parse 3.13",
+    )
     def test_completions_have_type(self):
         """
         Lets make sure matchers provide completion type.
@@ -507,7 +551,11 @@ class TestCompleter(unittest.TestCase):
             c = next(completions)  # should be `%time` or similar
         assert c.type == "magic", "Type of magic was not assigned by completer"
 
-    @pytest.mark.xfail(reason="Known failure on jedi<=0.18.0")
+    @pytest.mark.xfail(
+        parse(version("jedi")) <= parse("0.18.0"),
+        reason="Known failure on jedi<=0.18.0",
+        strict=True,
+    )
     def test_deduplicate_completions(self):
         """
         Test that completions are correctly deduplicated (even if ranges are not the same)
@@ -531,6 +579,10 @@ class TestCompleter(unittest.TestCase):
         assert len(l) == 1, "Completions (Z.z<tab>) correctly deduplicate: %s " % l
         assert l[0].text == "zoo"  # and not `it.accumulate`
 
+    @pytest.mark.xfail(
+        sys.version_info.releaselevel in ("alpha",),
+        reason="Parso does not yet parse 3.13",
+    )
     def test_greedy_completions(self):
         """
         Test the capability of the Greedy completer.
@@ -556,7 +608,7 @@ class TestCompleter(unittest.TestCase):
                 ip.Completer.use_jedi = True
                 with provisionalcompleter():
                     completions = ip.Completer.completions(line, cursor_pos)
-                self.assertIn(completion, completions)
+                self.assertIn(completion, list(completions))
 
         with provisionalcompleter():
             _(
@@ -573,7 +625,6 @@ class TestCompleter(unittest.TestCase):
                 "Should have completed on a[0].r: %s",
                 Completion(5, 6, "real"),
             )
-
             _(
                 "a[0].from_",
                 10,
@@ -584,7 +635,7 @@ class TestCompleter(unittest.TestCase):
             _(
                 "assert str.star",
                 14,
-                "str.startswith",
+                ".startswith",
                 "Should have completed on `assert str.star`: %s",
                 Completion(11, 14, "startswith"),
             )
@@ -595,6 +646,13 @@ class TestCompleter(unittest.TestCase):
                 "Should have completed on `d['a b'].str`: %s",
                 Completion(9, 12, "strip"),
             )
+            _(
+                "a.app",
+                4,
+                ".append",
+                "Should have completed on `a.app`: %s",
+                Completion(2, 4, "append"),
+            )
 
     def test_omit__names(self):
         # also happens to test IPCompleter as a configurable
@@ -609,8 +667,8 @@ class TestCompleter(unittest.TestCase):
         with provisionalcompleter():
             c.use_jedi = False
             s, matches = c.complete("ip.")
-            self.assertIn("ip.__str__", matches)
-            self.assertIn("ip._hidden_attr", matches)
+            self.assertIn(".__str__", matches)
+            self.assertIn("._hidden_attr", matches)
 
             # c.use_jedi = True
             # completions = set(c.completions('ip.', 3))
@@ -623,7 +681,7 @@ class TestCompleter(unittest.TestCase):
         with provisionalcompleter():
             c.use_jedi = False
             s, matches = c.complete("ip.")
-            self.assertNotIn("ip.__str__", matches)
+            self.assertNotIn(".__str__", matches)
             # self.assertIn('ip._hidden_attr', matches)
 
             # c.use_jedi = True
@@ -637,8 +695,8 @@ class TestCompleter(unittest.TestCase):
         with provisionalcompleter():
             c.use_jedi = False
             s, matches = c.complete("ip.")
-            self.assertNotIn("ip.__str__", matches)
-            self.assertNotIn("ip._hidden_attr", matches)
+            self.assertNotIn(".__str__", matches)
+            self.assertNotIn("._hidden_attr", matches)
 
             # c.use_jedi = True
             # completions = set(c.completions('ip.', 3))
@@ -648,7 +706,7 @@ class TestCompleter(unittest.TestCase):
         with provisionalcompleter():
             c.use_jedi = False
             s, matches = c.complete("ip._x.")
-            self.assertIn("ip._x.keys", matches)
+            self.assertIn(".keys", matches)
 
             # c.use_jedi = True
             # completions = set(c.completions('ip._x.', 6))
@@ -659,7 +717,7 @@ class TestCompleter(unittest.TestCase):
 
     def test_limit_to__all__False_ok(self):
         """
-        Limit to all is deprecated, once we remove it this test can go away. 
+        Limit to all is deprecated, once we remove it this test can go away.
         """
         ip = get_ipython()
         c = ip.Completer
@@ -670,7 +728,7 @@ class TestCompleter(unittest.TestCase):
         cfg.IPCompleter.limit_to__all__ = False
         c.update_config(cfg)
         s, matches = c.complete("d.")
-        self.assertIn("d.x", matches)
+        self.assertIn(".x", matches)
 
     def test_get__all__entries_ok(self):
         class A:
@@ -686,6 +744,46 @@ class TestCompleter(unittest.TestCase):
         words = completer.get__all__entries(A())
         self.assertEqual(words, [])
 
+    def test_completes_globals_as_args_of_methods(self):
+        ip = get_ipython()
+        c = ip.Completer
+        c.use_jedi = False
+        ip.ex("long_variable_name = 1")
+        ip.ex("a = []")
+        s, matches = c.complete(None, "a.sort(lo")
+        self.assertIn("long_variable_name", matches)
+
+    def test_completes_attributes_in_fstring_expressions(self):
+        ip = get_ipython()
+        c = ip.Completer
+        c.use_jedi = False
+
+        class CustomClass:
+            def method_one(self):
+                pass
+
+        ip.user_ns["custom_obj"] = CustomClass()
+
+        # Test completion inside f-string expressions
+        s, matches = c.complete(None, "f'{custom_obj.meth")
+        self.assertIn(".method_one", matches)
+
+    def test_completes_in_dict_expressions(self):
+        ip = get_ipython()
+        c = ip.Completer
+        c.use_jedi = False
+        ip.ex("class Test: pass")
+        ip.ex("test_obj = Test()")
+        ip.ex("test_obj.attribute = 'value'")
+
+        # Test completion in dictionary expressions
+        s, matches = c.complete(None, "d = {'key': test_obj.attr")
+        self.assertIn(".attribute", matches)
+
+        # Test global completion in dictionary expressions with dots
+        s, matches = c.complete(None, "d = {'k.e.y': Te")
+        self.assertIn("Test", matches)
+
     def test_func_kw_completions(self):
         ip = get_ipython()
         c = ip.Completer
@@ -1608,6 +1706,10 @@ class TestCompleter(unittest.TestCase):
             configure(None)
             _("--", ["completion_iter", "completion_list"])
 
+    @pytest.mark.xfail(
+        sys.version_info.releaselevel in ("alpha",),
+        reason="Parso does not yet parse 3.13",
+    )
     def test_matcher_suppression_with_jedi(self):
         ip = get_ipython()
         c = ip.Completer
@@ -1682,6 +1784,124 @@ class TestCompleter(unittest.TestCase):
             _(["completion_a"])
 
 
+@pytest.mark.parametrize(
+    "line,expected",
+    [
+        # Basic test cases
+        ("np.", "attribute"),
+        ("np.ran", "attribute"),
+        ("np.random.rand(np.random.ran", "attribute"),
+        ("np.random.rand(n", "global"),
+        ("d['k.e.y.'](ran", "global"),
+        ("d[0].k", "attribute"),
+        ("a = { 'a': np.ran", "attribute"),
+        ("n", "global"),
+        ("", "global"),
+        # Dots in string literals
+        ('some_var = "this is a string with a dot.', "global"),
+        ("text = 'another string with a dot.", "global"),
+        ('f"greeting {user.na', "attribute"),  # Cursor in f-string expression
+        ('t"welcome {guest.na', "attribute"),  # Cursor in t-string expression
+        ('f"hello {name} worl', "global"),  # Cursor in f-string outside expression
+        ('f"hello {{a.', "global"),
+        ('f"hello {{{a.', "attribute"),
+        # Backslash escapes in strings
+        ('var = "string with \\"escaped quote and a dot.', "global"),
+        ("escaped = 'single \\'quote\\' with a dot.", "global"),
+        # Multi-line strings
+        ('multi = """This is line one\nwith a dot.', "global"),
+        ("multi_single = '''Another\nmulti-line\nwith a dot.", "global"),
+        # Inline comments
+        ("x = 5  # This is a comment", "global"),
+        ("y = obj.method()  # Comment after dot.method", "global"),
+        # Hash symbol within string literals should not be treated as comments
+        ("d['#'] = np.", "attribute"),
+        # Nested parentheses with dots
+        ("complex_expr = (func((obj.method(param.attr", "attribute"),
+        ("multiple_nesting = {key: [value.attr", "attribute"),
+        # Numbers
+        ("3.", "global"),
+        ("3.14", "global"),
+        ("-42.14", "global"),
+        ("x = func(3.14", "global"),
+        ("x = func(a3.", "attribute"),
+        ("x = func(a3.12", "global"),
+        ("3.1.", "attribute"),
+        ("-3.1.", "attribute"),
+        ("(3).", "attribute"),
+        # Additional cases
+        ("", "global"),
+        ('str_with_code = "x.attr', "global"),
+        ('f"formatted {obj.attr', "attribute"),
+        ('f"formatted {obj.attr}', "global"),
+        ("dict_with_dots = {'key.with.dots': value.attr", "attribute"),
+        ("d[f'{a}']['{a.", "global"),
+    ],
+)
+def test_completion_context(line, expected):
+    """Test completion context"""
+    ip = get_ipython()
+    get_context = ip.Completer._determine_completion_context
+    result = get_context(line)
+    assert result.value == expected, f"Failed on input: '{line}'"
+
+
+@pytest.mark.xfail(reason="Completion context not yet supported")
+@pytest.mark.parametrize(
+    "line, expected",
+    [
+        ("f'{f'a.", "global"),  # Nested f-string
+        ("3a.", "global"),  # names starting with numbers or other symbols
+        ("$).", "global"),  # random things with dot at end
+    ],
+)
+def test_unsupported_completion_context(line, expected):
+    """Test unsupported completion context"""
+    ip = get_ipython()
+    get_context = ip.Completer._determine_completion_context
+    result = get_context(line)
+    assert result.value == expected, f"Failed on input: '{line}'"
+
+
+@pytest.mark.parametrize(
+    "setup,code,expected,not_expected",
+    [
+        ('a="str"; b=1', "(a, b.", [".bit_count", ".conjugate"], [".count"]),
+        ('a="str"; b=1', "(a, b).", [".count"], [".bit_count", ".capitalize"]),
+        ('x="str"; y=1', "x = {1, y.", [".bit_count"], [".count"]),
+        ('x="str"; y=1', "x = [1, y.", [".bit_count"], [".count"]),
+        ('x="str"; y=1; fun=lambda x:x', "x = fun(1, y.", [".bit_count"], [".count"]),
+    ],
+)
+def test_misc_no_jedi_completions(setup, code, expected, not_expected):
+    ip = get_ipython()
+    c = ip.Completer
+    ip.ex(setup)
+    with provisionalcompleter(), jedi_status(False):
+        matches = c.all_completions(code)
+        assert set(expected) - set(matches) == set(), set(matches)
+        assert set(matches).intersection(set(not_expected)) == set()
+
+
+@pytest.mark.parametrize(
+    "code,expected",
+    [
+        (" (a, b", "b"),
+        ("(a, b", "b"),
+        ("(a, b)", ""),  # trim always start by trimming
+        (" (a, b)", "(a, b)"),
+        (" [a, b]", "[a, b]"),
+        (" a, b", "b"),
+        ("x = {1, y", "y"),
+        ("x = [1, y", "y"),
+        ("x = fun(1, y", "y"),
+    ],
+)
+def test_trim_expr(code, expected):
+    c = get_ipython().Completer
+    assert c._trim_expr(code) == expected
+
+
 @pytest.mark.parametrize(
     "input, expected",
     [
