@@ -2,9 +2,10 @@
 """Test IPython.core.logger"""
 
 import os.path
-import pytest
 
-from IPython.utils.tempdir import TemporaryDirectory
+import pytest
+from tempfile import TemporaryDirectory
+
 
 def test_logstart_inaccessible_file():
     with pytest.raises(IOError):
@@ -20,7 +21,7 @@ def test_logstart_unicode():
         logfname = os.path.join(tdir, "test_unicode.log")
         _ip.run_cell("'abc€'")
         try:
-            _ip.magic("logstart -to %s" % logfname)
+            _ip.run_line_magic("logstart", "-to %s" % logfname)
             _ip.run_cell("'abc€'")
         finally:
             _ip.logger.logstop()
