@@ -6,12 +6,18 @@ import io
 import os
 import re
 import shlex
+import signal
 import sys
 import warnings
 from importlib import invalidate_caches
 from io import StringIO
 from pathlib import Path
+from time import sleep
+from threading import Thread
+from subprocess import CalledProcessError
 from textwrap import dedent
+from time import sleep
+from threading import Thread
 from unittest import TestCase, mock
 
 import pytest
@@ -1121,6 +1127,36 @@ def test_script_config():
     assert "whoda" in sm.magics["cell"]
 
 
+def _interrupt_after_1s():
+    sleep(1)
+    signal.raise_signal(signal.SIGINT)
+
+
+def test_script_raise_on_interrupt():
+    ip = get_ipython()
+
+    with pytest.raises(CalledProcessError):
+        thread = Thread(target=_interrupt_after_1s)
+        thread.start()
+        ip.run_cell_magic(
+            "script", f"{sys.executable}", "from time import sleep; sleep(2)"
+        )
+        thread.join()
+
+
+def test_script_do_not_raise_on_interrupt():
+    ip = get_ipython()
+
+    thread = Thread(target=_interrupt_after_1s)
+    thread.start()
+    ip.run_cell_magic(
+        "script",
+        f"--no-raise-error {sys.executable}",
+        "from time import sleep; sleep(2)",
+    )
+    thread.join()
+
+
 def test_script_out():
     ip = get_ipython()
     ip.run_cell_magic("script", f"--out output {sys.executable}", "print('hi')")
@@ -1225,6 +1261,37 @@ def test_script_defaults():
             assert cmd in ip.magics_manager.magics["cell"]
 
 
+async def test_script_streams_continiously(capsys):
+    ip = get_ipython()
+    # Windows is slow to start up a thread on CI
+    is_windows = os.name == "nt"
+    step = 3 if is_windows else 1
+    code = dedent(
+        f"""\
+    import time
+    for _ in range(3):
+        time.sleep({step})
+        print(".", flush=True, end="")
+    """
+    )
+
+    def print_numbers():
+        for i in range(3):
+            sleep(step)
+            print(i, flush=True, end="")
+
+    thread = Thread(target=print_numbers)
+    thread.start()
+    sleep(step / 2)
+    ip.run_cell_magic("script", f"{sys.executable}", code)
+    thread.join()
+
+    captured = capsys.readouterr()
+    # If the streaming was line-wise or broken
+    # we would get `012...`
+    assert captured.out == "0.1.2."
+
+
 @magics_class
 class FooFoo(Magics):
     """class with both %foo and %%foo magics"""
