@@ -6,11 +6,10 @@
 import inspect
 import sys
 
-# Third party
-import nose.tools as nt
-
 # Our own
 from IPython.testing import decorators as dec
+from IPython.testing.skipdoctest import skip_doctest
+from IPython.utils.text import dedent
 
 #-----------------------------------------------------------------------------
 # Utilities
@@ -59,6 +58,7 @@ def test_deliberately_broken2():
 
 # Verify that we can correctly skip the doctest for a function at will, but
 # that the docstring itself is NOT destroyed by the decorator.
+@skip_doctest
 def doctest_bad(x,y=1,**k):
     """A function whose doctest we need to skip.
 
@@ -90,10 +90,11 @@ def test_skip_dt_decorator():
     >>> 1+1
     3
     """
+
     # Fetch the docstring from doctest_bad after decoration.
     val = doctest_bad.__doc__
-    
-    nt.assert_equal(check,val,"doctest_bad docstrings don't match")
+
+    assert dedent(check) == dedent(val), "doctest_bad docstrings don't match"
 
 
 # Doctest skipping should work for class methods too
@@ -106,6 +107,7 @@ class FooClass(object):
     2
     """
 
+    @skip_doctest
     def __init__(self,x):
         """Make a FooClass.
 
@@ -117,6 +119,7 @@ class FooClass(object):
         print('Making a FooClass.')
         self.x = x
         
+    @skip_doctest
     def bar(self,y):
         """Example:
 
@@ -152,13 +155,14 @@ def test_skip_dt_decorator2():
 
 @dec.skip_linux
 def test_linux():
-    nt.assert_false(sys.platform.startswith('linux'),"This test can't run under linux")
+    assert sys.platform.startswith("linux") is False, "This test can't run under linux"
+
 
 @dec.skip_win32
 def test_win32():
-    nt.assert_not_equal(sys.platform,'win32',"This test can't run under windows")
+    assert sys.platform != "win32", "This test can't run under windows"
+
 
 @dec.skip_osx
 def test_osx():
-    nt.assert_not_equal(sys.platform,'darwin',"This test can't run under osx")
-
+    assert sys.platform != "darwin", "This test can't run under osx"
