@@ -10,15 +10,11 @@ This file is meant to be used by process.py
 #  the file COPYING, distributed as part of this software.
 #-----------------------------------------------------------------------------
 
-from __future__ import print_function
 
 # stdlib
 import os, sys, threading
 import ctypes, msvcrt
 
-# local imports
-from . import py3compat
-
 # Win32 API types needed for the API calls
 from ctypes import POINTER
 from ctypes.wintypes import HANDLE, HLOCAL, LPVOID, WORD, DWORD, BOOL, \
@@ -174,7 +170,7 @@ class AvoidUNCPath(object):
             os.system(cmd)
     """
     def __enter__(self):
-        self.path = py3compat.getcwd()
+        self.path = os.getcwd()
         self.is_unc_path = self.path.startswith(r"\\")
         if self.is_unc_path:
             # change to c drive (as cmd.exe cannot handle UNC addresses)
@@ -296,7 +292,7 @@ class Win32ShellCommandController(object):
             c_hstdin = None
             CloseHandle(c_hstdout)
             c_hstdout = None
-            if c_hstderr != None:
+            if c_hstderr is not None:
                 CloseHandle(c_hstderr)
                 c_hstderr = None
 
@@ -403,10 +399,10 @@ class Win32ShellCommandController(object):
         These functions are called from different threads (but not
         concurrently, because of the GIL).
         """
-        if stdout_func == None and stdin_func == None and stderr_func == None:
+        if stdout_func is None and stdin_func is None and stderr_func is None:
             return self._run_stdio()
 
-        if stderr_func != None and self.mergeout:
+        if stderr_func is not None and self.mergeout:
             raise RuntimeError("Shell command was initiated with "
                     "merged stdin/stdout, but a separate stderr_func "
                     "was provided to the run() method")
@@ -421,7 +417,7 @@ class Win32ShellCommandController(object):
         threads.append(threading.Thread(target=self._stdout_thread,
                                     args=(self.hstdout, stdout_func)))
         if not self.mergeout:
-            if stderr_func == None:
+            if stderr_func is None:
                 stderr_func = stdout_func
             threads.append(threading.Thread(target=self._stdout_thread,
                                         args=(self.hstderr, stderr_func)))
@@ -541,7 +537,7 @@ class Win32ShellCommandController(object):
         if self.hstderr:
             CloseHandle(self.hstderr)
             self.hstderr = None
-        if self.piProcInfo != None:
+        if self.piProcInfo is not None:
             CloseHandle(self.piProcInfo.hProcess)
             CloseHandle(self.piProcInfo.hThread)
             self.piProcInfo = None
@@ -555,13 +551,13 @@ def system(cmd):
     Parameters
     ----------
     cmd : str
-      A command to be executed in the system shell.
+        A command to be executed in the system shell.
 
     Returns
     -------
     None : we explicitly do NOT return the subprocess status code, as this
     utility is meant to be used extensively in IPython, where any return value
-    would trigger :func:`sys.displayhook` calls.
+    would trigger : func:`sys.displayhook` calls.
     """
     with AvoidUNCPath() as path:
         if path is not None:
