@@ -4,7 +4,6 @@
 # Distributed under the terms of the BSD License.  The full license is in
 # the file COPYING, distributed as part of this software.
 #*****************************************************************************
-from __future__ import absolute_import
 
 """
 Color managing related utilities
@@ -22,5 +21,5 @@ class Colorable(Configurable):
     """
     A subclass of configurable for all the classes that have a `default_scheme`
     """
-    default_style=Unicode('lightbg').tag(config=True)
+    default_style=Unicode('LightBG').tag(config=True)
 
