@@ -12,18 +12,12 @@ import atexit
 import os
 import sys
 import tempfile
-import warnings
 from pathlib import Path
 from warnings import warn
 
 from IPython.utils.decorators import undoc
 from .capture import CapturedIO, capture_output
 
-# setup stdin/stdout/stderr to sys.stdin/sys.stdout/sys.stderr
-devnull = open(os.devnull, "w", encoding="utf-8")
-atexit.register(devnull.close)
-
-
 class Tee(object):
     """A class to duplicate an output stream to stdout/err.
 
@@ -81,6 +75,8 @@ class Tee(object):
         if not self._closed:
             self.close()
 
+    def isatty(self):
+        return False
 
 def ask_yes_no(prompt, default=None, interrupt=None):
     """Asks a question and returns a boolean (y/n) answer.
